from PyPDF2 import PdfFileReader, PdfFileWriter
from PyPDF2.generic import BooleanObject, NameObject, IndirectObject
from sys import argv

from . import utils

from .utils import ConfigRetriever


class PdfFieldFiller(PdfFileWriter):
    
    def __init__(self, configFile):
        super().__init__()
        config = ConfigRetriever(configFile)
        self.sourceFile = config.getSourceFilePath()
        self.targetFile = config.getTargetFilePath()
        self.fields = config.getFieldDict()
        self.metadata = config.getMetaDataDict()

    def _setMetaData(self, **info):
        self.metadata = info
    
    def _setFields(self, **fields):
        # If you need to set specific logic for certain fields, override this function
        self.fields = fields

    def _setNeedAppearanceWriter(self):
        # See 12.7.2 and 7.7.2 for more information: http://www.adobe.com/content/dam/acom/en/devnet/acrobat/pdfs/PDF32000_2008.pdf
        try:
            catalog = self._root_object
            # get the AcroForm tree
            if "/AcroForm" not in catalog:
                self._root_object.update({NameObject("/AcroForm"): IndirectObject(len(self._objects), 0, self)})

            need_appearances = NameObject("/NeedAppearances")
            self._root_object["/AcroForm"][need_appearances] = BooleanObject(True)
        except Exception as e:
            print('setNeedAppearanceWriter() catch : ', repr(e))
        finally:
            return self

    def post(self):
        with open(self.sourceFile, 'rb') as f:
            sourcePDF = PdfFileReader(f, strict=False)
            if "/AcroForm" in sourcePDF.trailer["/Root"]:
                sourcePDF.trailer["/Root"]["/AcroForm"].update({NameObject("/NeedAppearances"): BooleanObject(True)})
        
            self._setNeedAppearanceWriter()
            if "/AcroForm" in self._root_object:
                self._root_object["/AcroForm"].update({NameObject("/NeedAppearances"): BooleanObject(True)})
            
            self.appendPagesFromReader(sourcePDF)
            
            if self.metadata:
                metadata = dict(map(lambda x: (f'/{x[0].title()}', x[1]), self.metadata.items()))
                self.addMetadata(metadata)
            
            self.updatePageFormFieldValues(self.getPage(0), self.fields)
            
            with open(self.targetFile, 'wb') as f:
                self.write(f)

if __name__ == '__main__':
    main(argv[1])


import re
import os
import setuptools

longDescription = """
It's supercalifragilisticexpialidocious
Even though the sound of it
Is something quite atrocious
If you say it loud enough
You'll always sound precocious
Supercalifragilisticexpialidocious
"""

# path constants
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

# list of required dependencies
REQS_PATH = os.path.join(PROJECT_ROOT, 'requirements.txt')

with open(REQS_PATH, 'r') as rf:
    reqs = rf.read().splitlines()

# generate the version number
VERSIONFILE="PdfFieldFiller/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError(f"Unable to find version string in {VERSIONFILE}.")

setuptools.setup(
        name='PdfFieldFiller',
        version=verstr,
        description='"PDF Form Filling Tool"',
        long_description=longDescription,
        author="Alex Buitron",
        author_email="alex@buitron.com",
        classifiers = [
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Programming Language :: Python :: 3",
            "Operating System :: OS Independent",
            "Topic :: Software Development :: Libraries :: Python Modules",
            ],
        python_requires='>=3.6',
        install_requires=reqs,
        packages=["PdfFieldFiller"],
    )


from PyPDF2 import PdfFileReader
from sys import argv

def main(filePath):
    pdf = PdfFilePrinter(filePath)
    pdf.peak()

class PdfFilePrinter(PdfFileReader):
    def __init__(self, filePath):
        super().__init__(filePath)
        stream = open(filePath, 'rb')
        self.read(stream)
        self.stream = stream

    def peak(self):
        # pdf = open(self.filePath, 'rb')
            # pdf = self(f)
        info = self.getDocumentInfo()
        fields = self.getFields()
        metadata = self.getXmpMetadata()
        self.stream.close()
        
        if not fields:
            preformatFields = ['NA']
        else:
            preformatFields = [f'\n\t\t\t{list(fields.keys())[i]}' if i % 5 == 0 and i != 0  else list(fields.keys())[i] for i in range(len(fields))]
        
        stdout = f"""
        title:      {info.title}
        subject:    {info.subject}
        author:     {info.author}
        creator:    {info.creator}
        producer:   {info.producer}

        metadata:   {metadata}
        fields:     {','.join(preformatFields)}
        """ 
        print(stdout)


if __name__ == '__main__':
    main(argv[1])


from configparser import ConfigParser
from os import path

class ConfigRetriever():
    def __init__(self, filePath):
        self.config = ConfigParser()
        self.config.read(filePath)

    def getSourceFilePath(self):
        return self.config['File'].get('source_path') + self.config['File'].get('source_name')

    def getTargetFilePath(self):
        return self.config['File'].get('target_path') + self.config['File'].get('target_name')

    def getFieldDict(self):
        return dict(self.config['Fillable Fields'])

    def getMetaDataDict(self):
        return dict(self.config['PDF Information'])

if __name__ == '__main__':
    print('\nTesting')
    config = ConfigRetriever(path.join('..', 'etc', 'sample_pdf.ini'))
    print(config.getMetaDataDict())